#!/bin/bash

function run_command
{
	command=$1
	echo $command
	eval $command
}

# the pipeline

echo `initializing ...`

#INPUT data

#Sample A gzipped fastq files
#FASTQA1="../data/pep1-1/530_216A_ATTCCT_L001_R1_0*fastq.gz" #example for multiple FASTQ filled use * or simply space separated lists. Has to be in the same order for both pairs
FASTQA1=../../../data/DRR001787/DRR001787_1.fastq.gz
FASTQA2=../../../data/DRR001787/DRR001787_2.fastq.gz
#Sample B gzipped fastq files
FASTQB2=../../../data/DRR001788/DRR001788_2.fastq.gz
FASTQB1=../../../data/DRR001788/DRR001788_1.fastq.gz

OUTPREFIXA=prefix_A
OUTPREFIXB=prefix_B

#parameters

SCRIPTS=/data/kmc_tools_tests_paper/bin/NIKS/scripts #<NIKS script folder>
CORES=8 #number of cores to use for blast searches
KMERSIZE=31 # RD:61
MAXCUTOFF=100 # maximum value for a cutoff from the histogram



LMERSIZE=25 # l-mer for seed generation RD:55
MINSIZE=21 # minimum number of kmers to make up a seed (minlength= MINSIZE+KMERSIZE-1) RD:51
HIGHBOUNDARY=10000 # seeds with a total coverage of this many kmers are discarded
TOLERANCE=10 #tolerance in seed pairing

JAVA=java

KMERUTILS=${SCRIPTS}/kmerUtils.jar



# 1. kmerize
#  < fastq-files, kmer-length, outprefix
#  > kmer-distribution
#

#JELLYFISH

#A

mkdir ${OUTPREFIXA}_kmers

echo `date` "Counting kmers A ..." 

run_command "/usr/bin/time -v taskset -c 0-$(($CORES-1)) jellyfish count -C -o ${OUTPREFIXA}_kmers/tmp -m ${KMERSIZE} -t ${CORES} -s 10G <( zcat ${FASTQA1} ${FASTQA2} )" #change if not gzipped

#if the hash size was too small... do a merge
COUNT=$(ls ${OUTPREFIXA}_kmers/tmp* |wc -l)

echo `date` "merging ${COUNT} datasets A ..."

if [ $COUNT -eq 1 ]
then
 mv ${OUTPREFIXA}_kmers/tmp_0 ${OUTPREFIXA}_kmers_jellyfish
else
 run_command "/usr/bin/time -v taskset -c 0-$(($CORES-1)) jellyfish merge -o ${OUTPREFIXA}_kmers_jellyfish ${OUTPREFIXA}_kmers/tmp*"
fi
rm -rf ${OUTPREFIXA}_kmers

echo `date` "Calculatng hist A ..."

run_command "/usr/bin/time -v taskset -c 0-$(($CORES-1)) jellyfish histo -f -o ${OUTPREFIXA}.kmers.hist.csv -t ${CORES} ${OUTPREFIXA}_kmers_jellyfish"
awk '{print $2"\t"$1}' ${OUTPREFIXA}.kmers.hist.csv > ${OUTPREFIXA}_tmp
mv ${OUTPREFIXA}_tmp ${OUTPREFIXA}.kmers.hist.csv

#B

mkdir ${OUTPREFIXB}_kmers

echo `date` "Counting kmers B ..."

run_command "/usr/bin/time -v taskset -c 0-$(($CORES-1)) jellyfish count -C -o ${OUTPREFIXB}_kmers/tmp -m ${KMERSIZE} -t ${CORES} -s 10G <( zcat ${FASTQB1} ${FASTQB2} )"

#if the hash size was too small... do a merge
COUNT=$(ls ${OUTPREFIXB}_kmers/tmp* |wc -l)

echo `date` "merging ${COUNT} datasets B ..."

if [ $COUNT -eq 1 ]
then
 mv ${OUTPREFIXB}_kmers/tmp_0 ${OUTPREFIXB}_kmers_jellyfish
else
 run_command "/usr/bin/time -v taskset -c 0-$(($CORES-1)) jellyfish merge -o ${OUTPREFIXB}_kmers_jellyfish ${OUTPREFIXB}_kmers/tmp*"
fi
rm -rf ${OUTPREFIXB}_kmers

echo `date` "Calculatng hist B ..."

run_command "/usr/bin/time -v taskset -c 0-$(($CORES-1)) jellyfish histo -f -o ${OUTPREFIXB}.kmers.hist.csv -t ${CORES} ${OUTPREFIXB}_kmers_jellyfish"
awk '{print $2"\t"$1}' ${OUTPREFIXB}.kmers.hist.csv > ${OUTPREFIXB}_tmp
mv ${OUTPREFIXB}_tmp ${OUTPREFIXB}.kmers.hist.csv


# 2. find cutoff values
#    < kmer dist
#    > cutoff values

#A
CUTOFFA=$(awk -v MAXCUTOFF=${MAXCUTOFF} -f ${SCRIPTS}/histCutoffPipe.awk ${OUTPREFIXA}.kmers.hist.csv)

echo $CUTOFFA > ${OUTPREFIXA}_cutoff.csv

#B
CUTOFFB=$(awk -v MAXCUTOFF=${MAXCUTOFF} -f ${SCRIPTS}/histCutoffPipe.awk ${OUTPREFIXB}.kmers.hist.csv)

echo $CUTOFFB > ${OUTPREFIXB}_cutoff.csv

# 3. contrast kmer-distributions
#  < cutoff values
#  > unique kmers
#

#JELLYFISH
run_command "/usr/bin/time -v taskset -c 0-$(($CORES-1)) bash ${SCRIPTS}/mergeCountFilesMarkFilterJellyfish.sh ${SCRIPTS} ${OUTPREFIXA}_kmers_jellyfish ${OUTPREFIXB}_kmers_jellyfish ${CUTOFFA} ${CUTOFFA} ${CUTOFFB} ${CUTOFFB} ${OUTPREFIXA}_unique.kmerDiff ${OUTPREFIXB}_unique.kmerDiff"



#We only perform above stages
exit



#REST OF CALCULATIONS
# 4. generate seeds and remove highly repetitive sequences
#  < unique kmers, lmer-length (% of kmer-length??), minSize, high boundary
#  > seeds
#

echo `date` "Generating seeds A ..."

#A
${JAVA} -Xmx64G -Xms64G -XX:-UseGCOverheadLimit -jar ${KMERUTILS} generateSeeds3 ${OUTPREFIXA}_unique.kmerDiff ${OUTPREFIXB}_unique.kmerDiff 0 ${MINSIZE} ${LMERSIZE} ${OUTPREFIXA}_seedsRep > /dev/null  #|gzip -c > ${OUTPREFIXA}_seedsRep_endCondition.csv.gz
awk -f ${SCRIPTS}/longToInfo.awk ${OUTPREFIXA}_seedsRep_long.txt |gzip -c> ${OUTPREFIXA}_seedsRep_longTable.csv.gz
gunzip -c ${OUTPREFIXA}_seedsRep_longTable.csv.gz| awk -v hb=${HIGHBOUNDARY} '$3<hb && $4<hb {print $1}' > ${OUTPREFIXA}_seeds.gi
${JAVA} -jar ${KMERUTILS} sub ${OUTPREFIXA}_seedsRep.fa ${OUTPREFIXA}_seeds.gi > ${OUTPREFIXA}_seeds.fa

echo `date` "Generating seeds B ..."

#B
${JAVA} -Xmx64G -Xms64G -XX:-UseGCOverheadLimit -jar ${KMERUTILS} generateSeeds3 ${OUTPREFIXB}_unique.kmerDiff ${OUTPREFIXA}_unique.kmerDiff 0 ${MINSIZE} ${LMERSIZE} ${OUTPREFIXB}_seedsRep > /dev/null  #|gzip -c > ${OUTPREFIXB}_seedsRep_endCondition.csv.gz
awk -f ${SCRIPTS}/longToInfo.awk ${OUTPREFIXB}_seedsRep_long.txt |gzip -c> ${OUTPREFIXB}_seedsRep_longTable.csv.gz
gunzip -c ${OUTPREFIXB}_seedsRep_longTable.csv.gz| awk -v hb=${HIGHBOUNDARY} '$3<hb && $4<hb {print $1}' > ${OUTPREFIXB}_seeds.gi
${JAVA} -jar ${KMERUTILS} sub ${OUTPREFIXB}_seedsRep.fa ${OUTPREFIXB}_seeds.gi > ${OUTPREFIXB}_seeds.fa


# 4.1. clean reverse complement
#    < seeds
#    > filtered seeds
#

### remove multiple hitting sequences here

#Find the smaller set and filter this. Identify the seeds in the other set which could match to the first filtered set. Filter those
if [ $( grep -c '>' ${OUTPREFIXA}_seeds.fa ) -lt $( grep -c '>' ${OUTPREFIXB}_seeds.fa ) ]
then
 #start with A
 PREFIX1=$OUTPREFIXA
 PREFIX2=$OUTPREFIXB
else
 #start with B
 PREFIX1=$OUTPREFIXB
 PREFIX2=$OUTPREFIXA
fi

echo `date` "Cleaning small set ..."

makeblastdb -in ${PREFIX1}_seeds.fa -dbtype nucl
blastn -query ${PREFIX1}_seeds.fa -db ${PREFIX1}_seeds.fa -num_threads ${CORES} -dust no -outfmt '6 std qlen slen' |${JAVA} -XX:-UseGCOverheadLimit -jar ${KMERUTILS} blastRevCompFilterPipe ${PREFIX1}_seeds.fa > ${PREFIX1}_seeds_filtered.fa

echo `date` "Reducing large set ..."
#prefilter second data set
makeblastdb -in ${PREFIX2}_seeds.fa -dbtype nucl
blastn -num_threads ${CORES} -query ${PREFIX1}_seeds_filtered.fa -db ${PREFIX2}_seeds.fa -outfmt 6 -task blastn | cut -f 2 | sort -u | ${JAVA} -jar ${KMERUTILS} subPipe ${PREFIX2}_seeds.fa > ${PREFIX2}_seeds_toUse.fa

echo `date` "Cleaning reduced large set ..."

makeblastdb -in ${PREFIX2}_seeds_toUse.fa -dbtype nucl
blastn -num_threads ${CORES} -query ${PREFIX2}_seeds_toUse.fa -db ${PREFIX2}_seeds_toUse.fa -dust no -outfmt '6 std qlen slen' |${JAVA} -XX:-UseGCOverheadLimit -jar ${KMERUTILS} blastRevCompFilterPipe ${PREFIX2}_seeds_toUse.fa > ${PREFIX2}_seeds_filtered.fa

# 5. contrast seeds (BLASTN -task blastn) remove sequences with multiple hits
#

echo `date` "Contrasting seeds A ..."
 
#A
makeblastdb -in ${OUTPREFIXB}_seeds_filtered.fa -dbtype nucl
blastn -num_threads ${CORES} -query ${OUTPREFIXA}_seeds_filtered.fa -db ${OUTPREFIXB}_seeds_filtered.fa -task blastn -dust no -outfmt '6 std qseq sseq'|gzip -c > ${OUTPREFIXA}_seeds_filtered_inOther_blast.csv.gz

echo `date` "Contrasting seeds B ..."

#B
makeblastdb -in ${OUTPREFIXA}_seeds_filtered.fa -dbtype nucl
blastn -num_threads ${CORES} -query ${OUTPREFIXB}_seeds_filtered.fa -db ${OUTPREFIXA}_seeds_filtered.fa -task blastn -dust no -outfmt '6 std qseq sseq'| gzip -c > ${OUTPREFIXB}_seeds_filtered_inOther_blast.csv.gz

# 5.1. SNPs
#    < seeds, tolerance
#    > candidate seed pairs, annotation file
#

#A

gunzip -c ${OUTPREFIXA}_seeds_filtered_inOther_blast.csv.gz | ${JAVA} -jar ${KMERUTILS} blastSNPFilterPipe ${KMERSIZE} ${TOLERANCE} > ${OUTPREFIXA}_SNP_blastRep.csv

#B

gunzip -c ${OUTPREFIXB}_seeds_filtered_inOther_blast.csv.gz | ${JAVA} -jar ${KMERUTILS} blastSNPFilterPipe ${KMERSIZE} ${TOLERANCE} > ${OUTPREFIXB}_SNP_blastRep.csv

${JAVA} -jar ${KMERUTILS} blastRemoveRepetitive ${OUTPREFIXA}_SNP_blastRep.csv ${OUTPREFIXB}_SNP_blastRep.csv ${OUTPREFIXA}_SNP_blast.csv ${OUTPREFIXB}_SNP_blast.csv


# 5.2. InDels
#    < seeds, ???
#    > candidate seed pairs???, annotation file
#

# 5.3. Merge
#    < annotations files
#    > merged annotations file
#

#A
cat ${OUTPREFIXA}_SNP_blast.csv > ${OUTPREFIXA}_annotated_blast.csv

#B
cat ${OUTPREFIXB}_SNP_blast.csv > ${OUTPREFIXB}_annotated_blast.csv

# 5.4. Extract fasta file with all seeds
#    < seeds fasta file, all paired seeds
#    > candidate seeds
#

#A
cut -f 1 ${OUTPREFIXA}_SNP_blast.csv |sort -gu > ${OUTPREFIXA}_candidates.gi
${JAVA} -jar ${KMERUTILS} sub ${OUTPREFIXA}_seeds_filtered.fa ${OUTPREFIXA}_candidates.gi > ${OUTPREFIXA}_candidates.fa

#B
cut -f 1 ${OUTPREFIXB}_SNP_blast.csv |sort -gu > ${OUTPREFIXB}_candidates.gi
${JAVA} -jar ${KMERUTILS} sub ${OUTPREFIXB}_seeds_filtered.fa ${OUTPREFIXB}_candidates.gi > ${OUTPREFIXB}_candidates.fa


# 6. extend candidate seeds
#

# 6.1. kmerize candidate seeds
#    < candidate seeds
#    > candidate seeds kmers
#

#A
${JAVA} -jar ${KMERUTILS} kmerize ${OUTPREFIXA}_candidates.fa ${KMERSIZE} |sort -u >  ${OUTPREFIXA}_candidates.kmers

#B
${JAVA} -jar ${KMERUTILS} kmerize ${OUTPREFIXB}_candidates.fa ${KMERSIZE} |sort -u >  ${OUTPREFIXB}_candidates.kmers

# 6.2. extract reads
#    < fastq files, candidate seeds kmers
#    > subseted read files
#

echo `date` "Extracting reads A ..."

#A
${JAVA} -jar ${KMERUTILS} kmerCoverage_extractCoveredZ ${FASTQA1} ${FASTQA2} ${OUTPREFIXA}_candidates.kmers 0 ${OUTPREFIXA}_candidates_reads

#when FASTQ files are not gzipped:
#${JAVA} -jar ${KMERUTILS} kmerCoverage_extractCovered ${FASTQA1} ${FASTQA2} ${OUTPREFIXA}_candidates.kmers 0 ${OUTPREFIXA}_candidates_reads

echo `date` "Extracting reads B ..."

#B
${JAVA} -jar ${KMERUTILS} kmerCoverage_extractCoveredZ ${FASTQB1} ${FASTQB2} ${OUTPREFIXB}_candidates.kmers 0 ${OUTPREFIXB}_candidates_reads

# 6.3. extend each seed
#    < candidate seed, subseted read files
#    > extended seed
#

echo `date` "Assembly A ..."

#A
mkdir ${OUTPREFIXA}_candidates
${JAVA} -jar ${KMERUTILS} split ${OUTPREFIXA}_candidates.fa ${OUTPREFIXA}_candidates/seq 1
for s in `ls ${OUTPREFIXA}_candidates/seq*.fa |grep -v extended`
do
bash ${SCRIPTS}/assemble.sh ${s} ${OUTPREFIXA}_candidates_reads_1.fastq ${OUTPREFIXA}_candidates_reads_2.fastq ${FASTQINSERT} ${KMERSIZE} ${SCRIPTS} &> /dev/null
done
cat ${OUTPREFIXA}_candidates/seq*extended.fa > ${OUTPREFIXA}_candidates_ext.fa

${JAVA} -jar ${KMERUTILS} lengths ${OUTPREFIXA}_candidates_ext.fa > ${OUTPREFIXA}_candidates_ext.lengths

tar czf ${OUTPREFIXA}_candidates_extData.tar.gz ${OUTPREFIXA}_candidates

echo `date` "Assembly B ..."

#B
mkdir ${OUTPREFIXB}_candidates
${JAVA} -jar ${KMERUTILS} split ${OUTPREFIXB}_candidates.fa ${OUTPREFIXB}_candidates/seq 1
for s in `ls ${OUTPREFIXB}_candidates/seq*.fa |grep -v extended`
do
bash ${SCRIPTS}/assemble.sh ${s} ${OUTPREFIXB}_candidates_reads_1.fastq ${OUTPREFIXB}_candidates_reads_2.fastq ${FASTQINSERT} ${KMERSIZE} ${SCRIPTS} &> /dev/null
done
cat ${OUTPREFIXB}_candidates/seq*extended.fa > ${OUTPREFIXB}_candidates_ext.fa

${JAVA} -jar ${KMERUTILS} lengths ${OUTPREFIXB}_candidates_ext.fa > ${OUTPREFIXB}_candidates_ext.lengths

tar czf ${OUTPREFIXB}_candidates_extData.tar.gz ${OUTPREFIXB}_candidates

# 7. annotate position in extended seed
#  < seeds, extended seed, seed annotation file
#  > extended seed annotation file
#

#A
makeblastdb -in ${OUTPREFIXA}_candidates_ext.fa -dbtype nucl
blastn -num_threads ${CORES} -query ${OUTPREFIXA}_candidates.fa -db ${OUTPREFIXA}_candidates_ext.fa -dust no -outfmt '6 std qlen slen' | ${JAVA} -jar ${KMERUTILS} blastTransferAnnotationPipe ${OUTPREFIXA}_annotated_blast.csv | ${JAVA} -jar ${KMERUTILS} blastMutationEMSAnnotationPipe > ${OUTPREFIXA}_candidates_ext_annotation.csv

#B
makeblastdb -in ${OUTPREFIXB}_candidates_ext.fa -dbtype nucl
blastn -num_threads ${CORES} -query ${OUTPREFIXB}_candidates.fa -db ${OUTPREFIXB}_candidates_ext.fa -dust no -outfmt '6 std qlen slen' | ${JAVA} -jar ${KMERUTILS} blastTransferAnnotationPipe ${OUTPREFIXB}_annotated_blast.csv | ${JAVA} -jar ${KMERUTILS} blastMutationEMSAnnotationPipe > ${OUTPREFIXB}_candidates_ext_annotation.csv



#make mastertable

join -1 1 -2 1 <(gunzip -c ${OUTPREFIXA}_seedsRep_longTable.csv.gz |grep -Ff <(cut -f 1 ${OUTPREFIXA}_candidates.gi ) |sort -k1,1) <(cut -f 1 ${OUTPREFIXA}_candidates.gi |sort -k1,1) |awk '{print $1"\t"$3"\t"$4}' |sort -k1,1g > ${OUTPREFIXA}_candidates_counts.csv
join -1 1 -2 1 <(gunzip -c ${OUTPREFIXB}_seedsRep_longTable.csv.gz |grep -Ff <(cut -f 1 ${OUTPREFIXB}_candidates.gi ) |sort -k1,1) <(cut -f 1 ${OUTPREFIXB}_candidates.gi |sort -k1,1) |awk '{print $1"\t"$3"\t"$4}' |sort -k1,1g > ${OUTPREFIXB}_candidates_counts.csv

${JAVA} -jar ${KMERUTILS} leftjoin ${OUTPREFIXA}_SNP_blast.csv ${OUTPREFIXA}_candidates.gi ${OUTPREFIXA}_candidates_ext.lengths ${OUTPREFIXA}_candidates_ext_annotation.csv ${OUTPREFIXA}_candidates_counts.csv 0 |awk 'NR>1' > ${OUTPREFIXA}_summarySelf.csv
${JAVA} -jar ${KMERUTILS} leftjoin ${OUTPREFIXB}_candidates.gi ${OUTPREFIXB}_candidates_ext.lengths ${OUTPREFIXB}_candidates_ext_annotation.csv ${OUTPREFIXB}_candidates_counts.csv 0 |awk 'NR>1' > ${OUTPREFIXB}_summarySelf.csv

join -1 2 -2 1 <(sort -k 2,2 ${OUTPREFIXA}_summarySelf.csv) <(sort -k 1,1 ${OUTPREFIXB}_summarySelf.csv) |awk -vKMER=${KMERSIZE} -v OFS="\t" ' {print $2,$1,$4,$5,$6,$14,$15,$16,$17,$18,$19,$20,$21,$22,$23,$18+$22,$17+$23,$4-length($16)-2*(KMER-1)}'|awk -vOFS="\t" -vKMER=${KMERSIZE} 'BEGIN {print "A.id\tB.id\talignment length\tmismatches\tgaps\tA.length\tA.annotation\tA.EMS\tA.Acount\tA.Bcount\tB.length\tB.annotation\tB.EMS\tB.Acount\tB.Bcount\tmirror count\tsupport count\ttolerance\tquality"} {qual=""} $16<=KMER && $18>-4 {qual="high quality"} {print $0,qual}' > ${OUTPREFIXA}_summaryBOTH.csv

echo `date` "Done!"
  